%global major   0
%define libname %mklibname bcc %major
%define devname %mklibname bcc -d

%global __requires_exclude_from %{?__requires_exclude_from:%__requires_exclude_from|}^%{_docdir}/.*$

# luajit is not available for some architectures
%ifarch ppc64 ppc64le s390x riscv64
%bcond_with lua
%else
%bcond_without lua
%endif

%define srcname bcc

%ifarch x86_64 ppc64 ppc64le aarch64 s390x riscv64
%bcond_without libbpf_tools
%else
%bcond_with libbpf_tools
%endif

%bcond_with llvm_static

%if %{without llvm_static}
%global with_llvm_shared 1
%endif


Name:           bcc
Version:        0.31.0
Release:        %mkrel 2
Summary:        BPF Compiler Collection (BCC)
Group:          Development/Kernel
License:        Apache-2.0
URL:            https://github.com/iovisor/bcc
Source0:        %{url}/archive/v%{version}/%{name}-%{version}.tar.gz

# Arches will be included as upstream support is added and dependencies are
# satisfied in the respective arches
ExclusiveArch:  x86_64 %{power64} aarch64 s390x armv7hl riscv64

BuildRequires:  bison
BuildRequires:  cmake >= 2.8.7
BuildRequires:  flex
BuildRequires:  zip
BuildRequires:  libxml2-devel
BuildRequires:  python3-devel
BuildRequires:  python3-setuptools
BuildRequires:  pkgconfig(libdebuginfod)
BuildRequires:  llvm-devel
BuildRequires:  clang-devel
%if %{with llvm_static}
BuildRequires:  llvm-static
%endif
BuildRequires:  ncurses-devel
%if %{with lua}
BuildRequires:  luajit
BuildRequires:  pkgconfig(luajit)
%endif
BuildRequires:  pkgconfig(libbpf) >= 0.0.8

Requires:       tar
Recommends:     kernel-devel

Recommends:     %{name}-tools = %{version}-%{release}

%description
BCC is a toolkit for creating efficient kernel tracing and manipulation
programs, and includes several useful tools and examples. It makes use of
extended BPF (Berkeley Packet Filters), formally known as eBPF, a new feature
that was first added to Linux 3.15. BCC makes BPF programs easier to write,
with kernel instrumentation in C (and includes a C wrapper around LLVM), and
front-ends in Python and lua. It is suited for many tasks, including
performance analysis and network traffic control.


%package -n %libname
Summary:        Shared library for BPF Compiler Collection (BCC)
Group:          System/Libraries

%description -n %libname
Library for BCC.

%package -n %devname
Group:          Development/C
Summary:        Development files for BPF Compiler Collection (BCC)
Requires:       %{libname}%{?_isa} = %{version}-%{release}

%description -n %devname
The %{name}-devel package contains libraries and header files for developing
application that use BPF Compiler Collection (BCC).


%package doc
Summary:        Examples for BPF Compiler Collection (BCC)
Group:          Documentation
Recommends:     python3-%{name} = %{version}-%{release}
Recommends:     %{name}-lua = %{version}-%{release}
BuildArch:      noarch

%description doc
Examples for BPF Compiler Collection (BCC).


%package -n python3-%{name}
Summary:        Python3 bindings for BPF Compiler Collection (BCC)
Group:		Development/Python
Requires:       %{libname} = %{version}-%{release}
#BuildArch:      noarch

%description -n python3-%{name}
Python3 bindings for BPF Compiler Collection (BCC)


%if %{with lua}
%package lua
Summary:        Standalone tool to run BCC tracers written in Lua
Group:          Development/Kernel
Requires:       %{libname}%{?_isa} = %{version}-%{release}

%description lua
Standalone tool to run BCC tracers written in Lua.
%endif


%package tools
Summary:        Command line tools for BPF Compiler Collection (BCC)
Group:          Development/Kernel
Requires:       %libname = %{version}-%{release}
Requires:       python3-%{name} = %{version}-%{release}
Requires:       python3-netaddr

%description tools
Command line tools for BPF Compiler Collection (BCC).

%if %{with libbpf_tools}
%package -n libbpf-tools
Summary:        Command line libbpf tools for BPF Compiler Collection (BCC)
Group:          Development/Kernel
BuildRequires:  pkgconfig(libbpf)
BuildRequires:  bpftool

%description -n libbpf-tools
Command line libbpf tools for BPF Compiler Collection (BCC).
%endif

%prep
%autosetup -p1

sed -i 's|CFLAGS :=|CFLAGS +=|' libbpf-tools/Makefile


%build
%cmake -DCMAKE_BUILD_TYPE=RelWithDebInfo \
       -DREVISION_LAST=%{version} -DREVISION=%{version} -DPYTHON_CMD=python3 \
       -DCMAKE_USE_LIBBPF_PACKAGE:BOOL=TRUE -DENABLE_NO_PIE=OFF \
       %{?with_llvm_shared:-DENABLE_LLVM_SHARED=1}
%cmake_build

# It was discussed and agreed to package libbpf-tools with
# 'bpf-' prefix (https://github.com/iovisor/bcc/pull/3263)
# Installing libbpf-tools binaries in temp directory and
# renaming them in there and the install code will just
# take them.
%if %{with libbpf_tools}
pushd libbpf-tools;
make BPFTOOL=bpftool LIBBPF_OBJ=%{_libdir}/libbpf.so CFLAGS="%{optflags}" LDFLAGS="%{build_ldflags}"
make DESTDIR=./tmp-install prefix= install
(
    cd tmp-install/bin
    for file in *; do
        mv $file bpf-$file
    done
    # now fix the broken symlinks
    for file in `find . -type l`; do
        dest=$(readlink "$file")
        ln -s -f bpf-$dest $file
    done
)
popd
%endif

%install
%cmake_install

# Fix python shebangs
find %{buildroot}%{_datadir}/%{name}/{tools,examples} -type f -exec \
  sed -i -e '1s=^#!/usr/bin/python\([0-9.]\+\)\?$=#!%{__python3}=' \
         -e '1s=^#!/usr/bin/env python\([0-9.]\+\)\?$=#!%{__python3}=' \
         -e '1s=^#!/usr/bin/env bcc-lua$=#!/usr/bin/bcc-lua=' {} \;

# Move man pages to the right location
mkdir -p %{buildroot}%{_mandir}
mv %{buildroot}%{_datadir}/%{name}/man/* %{buildroot}%{_mandir}/
# Avoid conflict with other manpages
# https://bugzilla.redhat.com/show_bug.cgi?id=1517408
for i in `find %{buildroot}%{_mandir} -name "*.gz"`; do
  tname=$(basename $i)
  rename $tname %{name}-$tname $i
done
mkdir -p %{buildroot}%{_docdir}/%{name}
mv %{buildroot}%{_datadir}/%{name}/examples %{buildroot}%{_docdir}/%{name}/

# Delete old tools we don't want to ship
rm -rf %{buildroot}%{_datadir}/%{name}/tools/old/

# We cannot run the test suit since it requires root and it makes changes to
# the machine (e.g, IP address)
#%check

%if %{with libbpf_tools}
mkdir -p %{buildroot}/%{_sbindir}
# We cannot use `install` because some of the tools are symlinks and `install`
# follows those. Since all the tools already have the correct permissions set,
# we just need to copy them to the right place while preserving those
cp -a libbpf-tools/tmp-install/bin/* %{buildroot}/%{_sbindir}/
%endif

%files -n %libname
%doc README.md
%license LICENSE.txt
%{_libdir}/lib%{name}.so.%{major}{,.*}
%{_libdir}/libbcc_bpf.so.%{major}{,.*}

%files -n %devname
%exclude %{_libdir}/lib%{name}*.a
%{_libdir}/lib%{name}.so
%{_libdir}/libbcc_bpf.so
%{_libdir}/pkgconfig/lib%{name}.pc
%{_includedir}/%{name}/

%files -n python3-%{name}
%{python3_sitelib}/%{name}*

%files doc
%dir %{_docdir}/%{name}
%doc %{_docdir}/%{name}/examples/

%files tools
%dir %{_datadir}/%{name}
%{_datadir}/%{name}/tools/
%{_datadir}/%{name}/introspection/
%{_mandir}/man8/*

%if %{with lua}
%files lua
%{_bindir}/bcc-lua
%endif

%if %{with libbpf_tools}
%files -n libbpf-tools
%{_sbindir}/bpf-*
%endif


%changelog
* Tue Mar 04 2025 wally <wally> 0.31.0-2.mga10
+ Revision: 2160345
- rebuild for python3.13

* Thu Nov 14 2024 wally <wally> 0.31.0-1.mga10
+ Revision: 2115534
- new version 0.31.0

* Sat Nov 02 2024 wally <wally> 0.30.0-2.mga10
+ Revision: 2112211
- rebuild with new libdebuginfod splitted out in elfutils 0.192

* Fri Sep 27 2024 wally <wally> 0.30.0-1.mga10
+ Revision: 2101577
- build libbpf-tools with shared libbpf
+ tv <tv>
- 0.30.0

* Sun Mar 31 2024 daviddavid <daviddavid> 0.29.1-2.mga10
+ Revision: 2053485
- rebuild for new llvm/clang 17
- add some fedora patches

* Fri Dec 29 2023 papoteur <papoteur> 0.29.1-1.mga10
+ Revision: 2024170
- new version 0.29.1
- rebuild for python 3.12
+ tv <tv>
- 0.28.0

* Tue Nov 15 2022 tv <tv> 0.25.0-2.mga9
+ Revision: 1907476
- Fixes some conversion errors raised by clang 15 (treated as warning before)

* Thu Sep 22 2022 tv <tv> 0.25.0-1.mga9
+ Revision: 1891471
- 0.25
- Rebuild for llvm-15

* Mon Aug 15 2022 tv <tv> 0.24.0-9.mga9
+ Revision: 1878028
- Fix filelist (typo in previous commit)
+ neoclust <neoclust>
- Fix the requires as there is no bcc package

* Wed Aug 10 2022 wally <wally> 0.24.0-8.mga9
+ Revision: 1876803
- add upstream patch to fix build with new libbpf
- rebuild for libbpf with lib major 1

* Fri Jun 24 2022 papoteur <papoteur> 0.24.0-7.mga9
+ Revision: 1865933
- Mass Rebuild for dependencies

* Mon Apr 25 2022 tv <tv> 0.24.0-6.mga9
+ Revision: 1853155
- Rebase to the latest release version
- Fix cmake build
- Add explicit Requires: bcc for bcc-tools
- Remove deprecated python_provide macro

* Sat Apr 02 2022 tv <tv> 0.22.0-6.mga9
+ Revision: 1840408
- Re-enable libbpf_tools

* Fri Apr 01 2022 tv <tv> 0.22.0-5.mga9
+ Revision: 1839896
- Fix build with clang-14
- Rebuild with clang-14
+ umeabot <umeabot>
- Mageia 9 Mass Rebuild

* Thu Feb 17 2022 tv <tv> 0.22.0-3.mga9
+ Revision: 1778405
- Rebuild for python-3.10

* Wed Nov 03 2021 tv <tv> 0.22.0-2.mga9
+ Revision: 1753843
- Make python !noarch for lib deps

* Fri Oct 22 2021 tv <tv> 0.22.0-1.mga9
+ Revision: 1752631
- 0.22.0
- disable building tools as they fail to build

* Sun Jun 06 2021 tv <tv> 0.20.0-1.mga9
+ Revision: 1729964
- 0.20.0, build from standard sources
- Keeping kernel-devel as weak dependency
- Don't require bcc-tools by default (rhbz#1966953)
- Patch 10: Fix build
- no need to pull kernel-devel
- Make the Python 3 bindings package noarch
- add libbpf-tools package
+ wally <wally>
- filter generated requires from doc pkg

* Mon Mar 29 2021 tv <tv> 0.19.0-1.mga9
+ Revision: 1711768
- 0.19.0

* Sat Mar 20 2021 tv <tv> 0.18.0-3.mga9
+ Revision: 1705575
- rebuild for python-3.9.2

* Mon Mar 01 2021 tv <tv> 0.18.0-2.mga9
+ Revision: 1695510
- rebuild for clang-12

* Sat Jan 16 2021 wally <wally> 0.18.0-1.mga8
+ Revision: 1671988
- add patches from upstream to fix build
- remove unneeded linking fixes
+ tv <tv>
- 0.18.0

* Thu Nov 26 2020 tv <tv> 0.17.0-3.mga8
+ Revision: 1649610
- rebuild for llvm

* Mon Nov 02 2020 pterjan <pterjan> 0.17.0-2.mga8
+ Revision: 1641827
- Add upstream patch to only run test_usdt_args on supported architectures

* Fri Oct 30 2020 tv <tv> 0.17.0-1.mga8
+ Revision: 1640612
- 0.17.0

* Sat Sep 26 2020 pterjan <pterjan> 0.16.0-4.mga8
+ Revision: 1630613
- Make the python package not noarch as it requires arch specific package name (libname) so can not be installed on either 32 or 64 architectures depending where it was built
- Drop incorrect workaround preventing from building on 32 bits architectures

* Thu Sep 17 2020 tv <tv> 0.16.0-2.mga8
+ Revision: 1628004
- Enable build for armv7hl

* Mon Aug 24 2020 tv <tv> 0.16.0-1.mga8
+ Revision: 1618549
- 0.16.0

* Sat Aug 22 2020 tv <tv> 0.15.0-3.mga8
+ Revision: 1617904
- make python subpkg noarch

* Wed Jul 15 2020 tv <tv> 0.15.0-2.mga8
+ Revision: 1606362
- Reinstate a function needed by bpftrace
- really update to 0.15 tarball
- Drop llvm-static dependency
- https://docs.fedoraproject.org/en-US/packaging-guidelines/#_statically_linking_executables

* Tue Jun 23 2020 tv <tv> 0.15.0-1.mga8
+ Revision: 1598670
- 0.15.0

* Thu May 07 2020 tv <tv> 0.14.0-1.mga8
+ Revision: 1581534
- 0.14.0

* Sun Mar 22 2020 tv <tv> 0.13.0-3.mga8
+ Revision: 1558890
- rebuild for llvm-10.0.0

* Thu Mar 12 2020 tv <tv> 0.13.0-2.mga8
+ Revision: 1555823
- our kernel doesn't build libpf on aarch64...
- fix more shebangs
- new release
- run make if make -j fails
- link against libclang-cpp.so
- workaround buggy libbpf-devel not pulling  libbpf0
- New release
- Add libbpf support
+ umeabot <umeabot>
- Mageia 8 Mass Rebuild

* Fri Dec 27 2019 wally <wally> 0.11.0-2.mga8
+ Revision: 1470242
- add patch to fix linking
- build with new cmake macros
- update summaries
- fix python shebangs with pathfix.py
- add lib major to file list

* Mon Oct 14 2019 tv <tv> 0.11.0-1.mga8
+ Revision: 1453030
- despite upstream claims, upstream tarball is still incomplete
- new release
- rebuild for clang-9.0

* Wed Sep 18 2019 tv <tv> 0.10.0-4.mga8
+ Revision: 1443438
- rebuild for clang-9.0

* Wed Sep 11 2019 tv <tv> 0.10.0-3.mga8
+ Revision: 1439764
- rebuild for python-3.8

* Fri Aug 09 2019 tv <tv> 0.10.0-2.mga8
+ Revision: 1428398
- python3 subpkg cannot be noarch as it requires libified lib subpkg

* Tue Jul 23 2019 tv <tv> 0.10.0-1.mga8
+ Revision: 1423593
- new release

* Thu May 09 2019 tv <tv> 0.9.0-1.mga7
+ Revision: 1396953
- new release
+ neoclust <neoclust>
- imported package bcc


* Wed Aug 02 2017 Fedora Release Engineering <releng@fedoraproject.org> - 0.3.0-4
- Rebuilt for https://fedoraproject.org/wiki/Fedora_27_Binutils_Mass_Rebuild

* Wed Jul 26 2017 Fedora Release Engineering <releng@fedoraproject.org> - 0.3.0-3
- Rebuilt for https://fedoraproject.org/wiki/Fedora_27_Mass_Rebuild

* Thu Mar 30 2017 Igor Gnatenko <ignatenko@redhat.com> - 0.3.0-2
- Rebuild for LLVM4
- Trivial fixes in spec

* Fri Mar 10 2017 Rafael Fonseca <rdossant@redhat.com> - 0.3.0-1
- Rebase to new release.

* Fri Feb 10 2017 Fedora Release Engineering <releng@fedoraproject.org> - 0.2.0-3
- Rebuilt for https://fedoraproject.org/wiki/Fedora_26_Mass_Rebuild

* Tue Jan 10 2017 Rafael Fonseca <rdossant@redhat.com> - 0.2.0-2
- Fix typo

* Tue Nov 29 2016 Rafael Fonseca <rdossant@redhat.com> - 0.2.0-1
- Initial import
